@@ -260,6 +260,19 @@ GSList *test_filename_return (void)
 
 /* multiple output arguments */
 
+/**
+ * test_multi_double_args:
+ * @in:
+ * @one: (out): 
+ * @two: (out): 
+ */
+void
+test_multi_double_args (gdouble in, gdouble *one, gdouble *two)
+{
+  *one = in * 2;
+  *two = in * 3;
+}
+
 /**
  * test_utf8_out_out:
  * @out0: (out) (transfer full): a copy of "first"
@@ -1515,13 +1528,15 @@ test_sub_obj_unset_bare (TestSubObj *obj)
 
 /**
  * test_callback:
- * @callback: (scope call):
+ * @callback: (scope call) (allow-none):
  *
  **/
 int
 test_callback (TestCallback callback)
 {
-  return callback();
+    if (callback != NULL)
+        return callback();
+    return 0;
 }
 
 /**
@@ -1557,7 +1572,7 @@ test_callback_destroy_notify (TestCallbackUserData callback,
 
   retval = callback(user_data);
 
-  info = g_new(CallbackInfo, 1);
+  info = g_slice_new(CallbackInfo);
   info->callback = callback;
   info->notify = notify;
   info->user_data = user_data;
@@ -1584,11 +1599,11 @@ test_callback_thaw_notifications (void)
 
   for (node = notified_callbacks; node != NULL; node = node->next)
     {
-      CallbackInfo *info = (CallbackInfo *)node->data;
+      CallbackInfo *info = node->data;
       retval += info->callback (info->user_data);
       if (info->notify)
         info->notify (info->user_data);
-      g_free (info);
+      g_slice_free (CallbackInfo, info);
     }
 
   g_slist_free (notified_callbacks);
@@ -1597,6 +1612,47 @@ test_callback_thaw_notifications (void)
   return retval;
 }
 
+static GSList *async_callbacks = NULL;
+
+/**
+ * test_callback_async:
+ * @callback: (scope async):
+ *
+ **/
+void
+test_callback_async (TestCallbackUserData callback,
+                     gpointer user_data)
+{
+  CallbackInfo *info;
+
+  info = g_slice_new(CallbackInfo);
+  info->callback = callback;
+  info->user_data = user_data;
+
+  async_callbacks = g_slist_prepend(async_callbacks, info);
+}
+
+/**
+ * test_callback_thaw_async:
+ */
+int
+test_callback_thaw_async (void)
+{
+  int retval = 0;
+  GSList *node;
+
+  for (node = async_callbacks; node != NULL; node = node->next)
+    {
+      CallbackInfo *info = node->data;
+      retval = info->callback (info->user_data);
+      g_slice_free (CallbackInfo, info);
+    }
+
+  g_slist_free (async_callbacks);
+  async_callbacks = NULL;
+  return retval;
+}
+
 /**
  * test_callback_infinite:
  * @callback: (scope infinite):
