@@ -30,21 +30,28 @@
 
 struct _GIBaseInfo 
 {
+  /* Keep this part in sync with GIUnresolvedInfo below */
   gint type;
   gint ref_count;
   GIRepository *repository;
   GIBaseInfo *container;
 
+  /* Resolved specific */
+
   GTypelib *typelib;
   guint32 offset;
 };
 
 struct _GIUnresolvedInfo
 {
+  /* Keep this part in sync with GIBaseInfo above */
   gint type;
   gint ref_count;
+  GIRepository *repository;
   GIBaseInfo *container;
 
+  /* Unresolved specific */
+
   const gchar *name;
   const gchar *namespace;
 };
@@ -127,6 +134,7 @@ struct _GIArgInfo
 struct _GITypeInfo
 {
   GIBaseInfo base;
+  gboolean   is_embedded;
 };
 
 struct _GIUnionInfo
@@ -147,7 +155,10 @@ g_info_new_full (GIInfoType     type,
 
   g_return_val_if_fail (container != NULL || repository != NULL, NULL);
 
-  info = g_new0 (GIBaseInfo, 1);
+  if (type == GI_INFO_TYPE_TYPE)
+    info = (GIBaseInfo *)g_new0 (GITypeInfo, 1);
+  else
+    info = g_new0 (GIBaseInfo, 1);
 
   info->ref_count = 1;
   info->type = type;
@@ -196,6 +207,7 @@ g_info_from_entry (GIRepository *repository,
 
 	  unresolved->type = GI_INFO_TYPE_UNRESOLVED;
 	  unresolved->ref_count = 1;
+	  unresolved->repository = g_object_ref (repository);
 	  unresolved->container = NULL;
 	  unresolved->name = name;
 	  unresolved->namespace = namespace;
@@ -542,6 +554,26 @@ g_base_info_get_typelib (GIBaseInfo *info)
   return info->typelib;
 }
 
+/*
+ * g_base_info_equal:
+ * @info1: A #GIBaseInfo
+ * @info2: A #GIBaseInfo
+ *
+ * Compare two #GIBaseInfo.
+ *
+ * Using pointer comparison is not practical since many functions return
+ * different instances of #GIBaseInfo that refers to the same part of the
+ * TypeLib; use this function instead to do #GIBaseInfo comparisons.
+ *
+ * Return value: TRUE if and only if @info1 equals @info2.
+ */
+gboolean
+g_base_info_equal (GIBaseInfo *info1, GIBaseInfo *info2)
+{
+  /* Compare the TypeLib pointers, which are mmapped. */
+  return info1->typelib->data + info1->offset == info2->typelib->data + info2->offset;
+}
+
 /* GIFunctionInfo functions */
 const gchar *
 g_function_info_get_symbol (GIFunctionInfo *info)
@@ -635,9 +667,13 @@ g_type_info_new (GIBaseInfo    *container,
 		 guint32        offset)
 {
   SimpleTypeBlob *type = (SimpleTypeBlob *)&typelib->data[offset];
+  GITypeInfo *type_info;
 
-  return (GITypeInfo *) g_info_new (GI_INFO_TYPE_TYPE, container, typelib, 
+  type_info = (GITypeInfo *) g_info_new (GI_INFO_TYPE_TYPE, container, typelib, 
 				    (type->flags.reserved == 0 && type->flags.reserved2 == 0) ? offset : type->offset);
+  type_info->is_embedded = FALSE;
+
+  return type_info;
 }
 
 /**
@@ -868,7 +904,9 @@ g_type_info_get_tag (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
 
-  if (type->flags.reserved == 0 && type->flags.reserved2 == 0)
+  if (info->is_embedded)
+    return GI_TYPE_TAG_INTERFACE;
+  else if (type->flags.reserved == 0 && type->flags.reserved2 == 0)
     return type->flags.tag;
   else
     {
@@ -912,7 +950,11 @@ g_type_info_get_interface (GITypeInfo *info)
 {
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
-  
+
+  if (info->is_embedded)
+    return (GIBaseInfo *) g_info_new (type->offset, base, base->typelib,
+				      base->offset);
+
   if (!(type->flags.reserved == 0 && type->flags.reserved2 == 0))
     {
       InterfaceTypeBlob *blob = (InterfaceTypeBlob *)&base->typelib->data[base->offset];
@@ -1092,8 +1134,21 @@ GITypeInfo *
 g_field_info_get_type (GIFieldInfo *info)
 {
   GIBaseInfo *base = (GIBaseInfo *)info;
-  
-  return g_type_info_new (base, base->typelib, base->offset + G_STRUCT_OFFSET (FieldBlob, type));
+  Header *header = (Header *)base->typelib->data;
+  FieldBlob *blob = (FieldBlob *)&base->typelib->data[base->offset];
+  GITypeInfo *type_info;
+
+  if (blob->has_embedded_type)
+    {
+      type_info = (GITypeInfo *) g_info_new (GI_INFO_TYPE_TYPE,
+                                             (GIBaseInfo*)info, base->typelib,
+                                             base->offset + header->field_blob_size);
+      type_info->is_embedded = TRUE;
+    }
+  else
+    return g_type_info_new (base, base->typelib, base->offset + G_STRUCT_OFFSET (FieldBlob, type));
+
+  return type_info;
 }
 
 /* GIRegisteredTypeInfo functions */
@@ -1153,16 +1208,35 @@ g_struct_info_get_n_fields (GIStructInfo *info)
   return blob->n_fields;
 }
 
+static gint32
+g_struct_get_field_offset (GIStructInfo *info,
+			   gint         n)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  Header *header = (Header *)base->typelib->data;
+  guint32 offset = base->offset + header->struct_blob_size;
+  gint i;
+  FieldBlob *field_blob;
+
+  for (i = 0; i < n; i++)
+    {
+      field_blob = (FieldBlob *)&base->typelib->data[offset];
+      offset += header->field_blob_size;
+      if (field_blob->has_embedded_type)
+        offset += header->callback_blob_size;
+    }
+
+  return offset;
+}
+
 GIFieldInfo *
 g_struct_info_get_field (GIStructInfo *info,
 			 gint         n)
 {
   GIBaseInfo *base = (GIBaseInfo *)info;
-  Header *header = (Header *)base->typelib->data;  
  
   return (GIFieldInfo *) g_info_new (GI_INFO_TYPE_FIELD, base, base->typelib, 
-				     base->offset + header->struct_blob_size + 
-				     n * header->field_blob_size);
+				     g_struct_get_field_offset (info, n));
 }
 
 gint
@@ -1183,8 +1257,7 @@ g_struct_info_get_method (GIStructInfo *info,
   Header *header = (Header *)base->typelib->data;  
   gint offset;
 
-  offset = base->offset + header->struct_blob_size 
-    + blob->n_fields * header->field_blob_size 
+  offset = g_struct_get_field_offset (info, blob->n_fields)
     + n * header->function_blob_size;
   return (GIFunctionInfo *) g_info_new (GI_INFO_TYPE_FUNCTION, base, 
 					base->typelib, offset);
