@@ -247,6 +247,7 @@ g_function_info_invoke (GIFunctionInfo *info,
   if (ffi_prep_cif (&cif, FFI_DEFAULT_ABI, n_invoke_args, rtype, atypes) != FFI_OK)
     goto out;
 
+  g_return_val_if_fail (return_value, FALSE);
   ffi_call (&cif, func, return_value, args);
 
   if (local_error)
@@ -407,18 +408,26 @@ gi_cclosure_marshal_generic (GClosure *closure,
   atypes = g_alloca (sizeof (ffi_type *) * n_args);
   args =  g_alloca (sizeof (gpointer) * n_args);
 
-  if (G_CCLOSURE_SWAP_DATA (closure))
+  if (n_param_values > 0)
     {
-      atypes[n_args-1] = value_to_ffi_type (param_values + 0,  
-                                            &args[n_args-1]);
-      atypes[0] = &ffi_type_pointer;
-      args[0] = &closure->data;
+      if (G_CCLOSURE_SWAP_DATA (closure))
+        {
+          atypes[n_args-1] = value_to_ffi_type (param_values + 0,  
+                                                &args[n_args-1]);
+          atypes[0] = &ffi_type_pointer;
+          args[0] = &closure->data;
+        }
+      else
+        {
+          atypes[0] = value_to_ffi_type (param_values + 0, &args[0]);
+          atypes[n_args-1] = &ffi_type_pointer;
+          args[n_args-1] = &closure->data;
+        }
     }
   else
     {
-      atypes[0] = value_to_ffi_type (param_values + 0, &args[0]);
-      atypes[n_args-1] = &ffi_type_pointer;
-      args[n_args-1] = &closure->data;
+      atypes[0] = &ffi_type_pointer;
+      args[0] = &closure->data;
     }
 
   for (i = 1; i < n_args - 1; i++)
@@ -427,6 +436,7 @@ gi_cclosure_marshal_generic (GClosure *closure,
   if (ffi_prep_cif (&cif, FFI_DEFAULT_ABI, n_args, rtype, atypes) != FFI_OK)
     return;
 
+  g_return_val_if_fail (rvalue, FALSE);
   ffi_call (&cif, marshal_data ? marshal_data : cc->callback, rvalue, args);
 
   if (return_gvalue && G_VALUE_TYPE (return_gvalue))
