@@ -31,6 +31,7 @@
 #include "girepository.h"
 #include "gtypelib.h"
 #include "ginfo.h"
+#include "glib-compat.h"
 
 #include "config.h"
 
@@ -363,9 +364,6 @@ register_internal (GIRepository *repository,
       g_hash_table_insert (repository->priv->typelibs, key, (void *)typelib);
     }
 
-  if (typelib->modules == NULL)
-    typelib->modules = g_list_append(typelib->modules, g_module_open (NULL, 0));
-
   return namespace;
 }
 
@@ -1042,7 +1040,7 @@ compare_candidate_reverse (struct NamespaceVersionCandidadate *c1,
 static void
 free_candidate (struct NamespaceVersionCandidadate *candidate)
 {
-  g_mapped_file_free (candidate->mfile);
+  g_mapped_file_unref (candidate->mfile);
   g_free (candidate->path);
   g_free (candidate->version);
   g_free (candidate);
