@@ -31,7 +31,8 @@
 GIrModule *
 g_ir_module_new (const gchar *name, 
 		 const gchar *version,
-		 const gchar *shared_library)
+		 const gchar *shared_library,
+		 const gchar *c_prefix)
 {
   GIrModule *module;
   
@@ -43,9 +44,13 @@ g_ir_module_new (const gchar *name,
       module->shared_library = g_strdup (shared_library);
   else
       module->shared_library = NULL;
+  module->c_prefix = g_strdup (c_prefix);
   module->dependencies = NULL;
   module->entries = NULL;
 
+  module->include_modules = NULL;
+  module->aliases = NULL;
+
   return module;
 }
 
@@ -62,9 +67,97 @@ g_ir_module_free (GIrModule *module)
   g_list_free (module->entries);
   /* Don't free dependencies, we inherit that from the parser */
 
+  g_list_free (module->include_modules);
+
+  g_hash_table_destroy (module->aliases);
+  g_hash_table_destroy (module->disguised_structures);
+
   g_free (module);
 }
 
+static void
+add_alias_foreach (gpointer key,
+		   gpointer value,
+		   gpointer data)
+{
+  GIrModule *module = data;
+
+  g_hash_table_replace (module->aliases, g_strdup (key), g_strdup (value));
+}
+
+static void
+add_disguised_structure_foreach (gpointer key,
+				 gpointer value,
+				 gpointer data)
+{
+  GIrModule *module = data;
+
+  g_hash_table_replace (module->disguised_structures, g_strdup (key), value);
+}
+
+void
+g_ir_module_add_include_module (GIrModule  *module,
+				GIrModule  *include_module)
+{
+  module->include_modules = g_list_prepend (module->include_modules,
+					    include_module);
+
+  g_hash_table_foreach (include_module->aliases,
+			add_alias_foreach,
+			module);
+
+  g_hash_table_foreach (include_module->disguised_structures,
+			add_disguised_structure_foreach,
+			module);
+}
+
+struct AttributeWriteData
+{
+  guint count;
+  guchar *databuf;
+  GIrNode *node;
+  GHashTable *strings;
+  guint32 *offset;
+  guint32 *offset2;
+};
+
+static void
+write_attribute (gpointer key, gpointer value, gpointer datap)
+{
+  struct AttributeWriteData *data = datap;
+  guint32 old_offset = *(data->offset);
+  AttributeBlob *blob = (AttributeBlob*)&(data->databuf[old_offset]);
+
+  *(data->offset) += sizeof (AttributeBlob);
+
+  blob->offset = data->node->offset;
+  blob->name = write_string ((const char*) key, data->strings, data->databuf, data->offset2);
+  blob->value = write_string ((const char*) value, data->strings, data->databuf, data->offset2);
+
+  data->count++;
+}
+
+static guint
+write_attributes (GIrModule *module,
+                   GIrNode   *node,
+                   GHashTable *strings,
+                   guchar    *data,
+                   guint32   *offset,
+                   guint32   *offset2)
+{
+  struct AttributeWriteData wdata;
+  wdata.count = 0;
+  wdata.databuf = data;
+  wdata.node = node;
+  wdata.offset = offset;
+  wdata.offset2 = offset2;
+  wdata.strings = strings;
+
+  g_hash_table_foreach (node->attributes, write_attribute, &wdata);
+
+  return wdata.count;
+}
+
 GTypelib *
 g_ir_module_build_typelib (GIrModule  *module,
 			     GList       *modules)
@@ -82,6 +175,7 @@ g_ir_module_build_typelib (GIrModule  *module,
   guint32 size, offset, offset2, old_offset;
   GHashTable *strings;
   GHashTable *types;
+  GList *offset_ordered_nodes;
   char *dependencies;
   guchar *data;
 
@@ -114,12 +208,13 @@ g_ir_module_build_typelib (GIrModule  *module,
   _g_irnode_init_stats ();
   strings = g_hash_table_new (g_str_hash, g_str_equal);
   types = g_hash_table_new (g_str_hash, g_str_equal);
+  offset_ordered_nodes = NULL;
   n_entries = g_list_length (module->entries);
 
   g_message ("%d entries (%d local), %d dependencies\n", n_entries, n_local_entries,
 	     g_list_length (module->dependencies));
   
-  dir_size = n_entries * 12;  
+  dir_size = n_entries * sizeof (DirEntry);
   size = header_size + dir_size;
 
   size += ALIGN_VALUE (strlen (module->name) + 1, 4);
@@ -129,14 +224,20 @@ g_ir_module_build_typelib (GIrModule  *module,
       GIrNode *node = e->data;
       
       size += g_ir_node_get_full_size (node);
+      size += g_ir_node_get_attribute_size (node);
+
+      /* Also reset the offset here */
+      node->offset = 0;
     }
 
   /* Adjust size for strings allocated in header below specially */
-  size += strlen (module->name);
+  size += ALIGN_VALUE (strlen (module->name) + 1, 4);
   if (module->shared_library) 
-    size += strlen (module->shared_library);
+    size += ALIGN_VALUE (strlen (module->shared_library) + 1, 4);
   if (dependencies != NULL)
-    size += strlen (dependencies);
+    size += ALIGN_VALUE (strlen (dependencies) + 1, 4);
+  if (module->c_prefix != NULL)
+    size += ALIGN_VALUE (strlen (module->c_prefix) + 1, 4);
 
   g_message ("allocating %d bytes (%d header, %d directory, %d entries)\n", 
 	  size, header_size, dir_size, size - header_size - dir_size);
@@ -146,13 +247,16 @@ g_ir_module_build_typelib (GIrModule  *module,
   /* fill in header */
   header = (Header *)data;
   memcpy (header, G_IR_MAGIC, 16);
-  header->major_version = 1;
+  header->major_version = 2;
   header->minor_version = 0;
   header->reserved = 0;
   header->n_entries = n_entries;
   header->n_local_entries = n_local_entries;
-  header->n_annotations = 0;
-  header->annotations = 0; /* filled in later */
+  header->n_attributes = 0;
+  header->attributes = 0; /* filled in later */
+  /* NOTE: When writing strings to the typelib here, you should also update
+   * the size calculations above.
+   */
   if (dependencies != NULL)
     header->dependencies = write_string (dependencies, strings, data, &header_size);
   else
@@ -163,25 +267,29 @@ g_ir_module_build_typelib (GIrModule  *module,
   header->shared_library = (module->shared_library?
                              write_string (module->shared_library, strings, data, &header_size)
                              : 0);
+  if (module->c_prefix != NULL)
+    header->c_prefix = write_string (module->c_prefix, strings, data, &header_size);
+  else
+    header->c_prefix = 0;
   header->directory = ALIGN_VALUE (header_size, 4);
-  header->entry_blob_size = 12;
-  header->function_blob_size = 16;
-  header->callback_blob_size = 12;
-  header->signal_blob_size = 12;
-  header->vfunc_blob_size = 16;
-  header->arg_blob_size = 12;
-  header->property_blob_size = 12;
-  header->field_blob_size = 12;
-  header->value_blob_size = 12;
-  header->constant_blob_size = 20;
-  header->error_domain_blob_size = 16;
-  header->annotation_blob_size = 12;
-  header->signature_blob_size = 8;
-  header->enum_blob_size = 20;
-  header->struct_blob_size = 20;
-  header->object_blob_size = 32;
-  header->interface_blob_size = 28;
-  header->union_blob_size = 28;
+  header->entry_blob_size = sizeof (DirEntry);
+  header->function_blob_size = sizeof (FunctionBlob);
+  header->callback_blob_size = sizeof (CallbackBlob);
+  header->signal_blob_size = sizeof (SignalBlob);
+  header->vfunc_blob_size = sizeof (VFuncBlob);
+  header->arg_blob_size = sizeof (ArgBlob);
+  header->property_blob_size = sizeof (PropertyBlob);
+  header->field_blob_size = sizeof (FieldBlob);
+  header->value_blob_size = sizeof (ValueBlob);
+  header->constant_blob_size = sizeof (ConstantBlob);
+  header->error_domain_blob_size = sizeof (ErrorDomainBlob);
+  header->attribute_blob_size = sizeof (AttributeBlob);
+  header->signature_blob_size = sizeof (SignatureBlob);
+  header->enum_blob_size = sizeof (EnumBlob);
+  header->struct_blob_size = sizeof (StructBlob);
+  header->object_blob_size = sizeof(ObjectBlob);
+  header->interface_blob_size = sizeof (InterfaceBlob);
+  header->union_blob_size = sizeof (UnionBlob);
 
   /* fill in directory and content */
   entry = (DirEntry *)&data[header->directory];
@@ -190,6 +298,7 @@ g_ir_module_build_typelib (GIrModule  *module,
 
   for (e = module->entries, i = 0; e; e = e->next, i++)
     {
+      GIrTypelibBuild build;
       GIrNode *node = e->data;
 
       if (strchr (node->name, '.'))
@@ -200,10 +309,17 @@ g_ir_module_build_typelib (GIrModule  *module,
       /* we picked up implicit xref nodes, start over */
       if (i == n_entries)
 	{
+	  GList *link;
 	  g_message ("Found implicit cross references, starting over");
 
 	  g_hash_table_destroy (strings);
 	  g_hash_table_destroy (types);
+
+	  /* Reset the cached offsets */
+	  for (link = offset_ordered_nodes; link; link = link->next)
+	    ((GIrNode *) link->data)->offset = 0;
+
+	  g_list_free (offset_ordered_nodes);
 	  strings = NULL;
 
 	  g_free (data);
@@ -233,8 +349,17 @@ g_ir_module_build_typelib (GIrModule  *module,
 	  entry->offset = offset;
 	  entry->name = write_string (node->name, strings, data, &offset2);
 
-	  g_ir_node_build_typelib (node, module, modules, 
-				     strings, types, data, &offset, &offset2);
+	  build.module = module;
+	  build.modules = modules;
+	  build.strings = strings;
+	  build.types = types;
+	  build.offset_ordered_nodes = offset_ordered_nodes;
+	  build.n_attributes = header->n_attributes;
+	  build.data = data;
+	  g_ir_node_build_typelib (node, NULL, &build, &offset, &offset2);
+
+	  offset_ordered_nodes = build.offset_ordered_nodes;
+	  header->n_attributes = build.n_attributes;
 
 	  if (offset2 > old_offset + g_ir_node_get_full_size (node))
 	    g_error ("left a hole of %d bytes\n", offset2 - old_offset - g_ir_node_get_full_size (node));
@@ -243,9 +368,23 @@ g_ir_module_build_typelib (GIrModule  *module,
       entry++;
     }
 
+  offset_ordered_nodes = g_list_reverse (offset_ordered_nodes);
+
+  g_message ("header: %d entries, %d attributes", header->n_entries, header->n_attributes);
+
   _g_irnode_dump_stats ();
 
-  header->annotations = offset2;
+  /* Write attributes after the blobs */
+  offset = offset2;
+  header->attributes = offset;
+  offset2 = offset + header->n_attributes * header->attribute_blob_size;
+
+  for (e = offset_ordered_nodes; e; e = e->next)
+    {
+      GIrNode *node = e->data;
+
+      write_attributes (module, node, strings, data, &offset, &offset2);
+    }
   
   g_message ("reallocating to %d bytes", offset2);
 
@@ -256,6 +395,7 @@ g_ir_module_build_typelib (GIrModule  *module,
 
   g_hash_table_destroy (strings);
   g_hash_table_destroy (types);
+  g_list_free (offset_ordered_nodes);
 
   return typelib;
 }
