@@ -279,6 +279,7 @@ g_ir_node_free (GIrNode *node)
 	
 	g_free (node->name);
 	g_ir_node_free ((GIrNode *)field->type);
+	g_ir_node_free ((GIrNode *)field->callback);
       }
       break;
 
@@ -508,7 +509,13 @@ g_ir_node_get_size (GIrNode *node)
       break;
 
     case G_IR_NODE_FIELD:
-      size = sizeof (FieldBlob);
+      {
+	GIrNodeField *field = (GIrNodeField *)node;
+
+        size = sizeof (FieldBlob);
+        if (field->callback)
+          size += g_ir_node_get_size ((GIrNode *)field->callback);
+      }
       break;
 
     case G_IR_NODE_CONSTANT:
@@ -797,7 +804,10 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 
 	size = sizeof (FieldBlob);
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
-	size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->type);	
+	if (field->callback)
+          size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->callback);
+	else
+          size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->type);
       }
       break;
 
@@ -1229,6 +1239,8 @@ serialize_type (GIrModule    *module,
     "uint32", 
     "int64", 
     "uint64", 
+    "short",
+    "ushort",
     "int",
     "uint",
     "long",
@@ -1237,11 +1249,10 @@ serialize_type (GIrModule    *module,
     "size",
     "float", 
     "double",
+    "time_t",
+    "GType",
     "utf8", 
     "filename",
-    "string",
-    "sequence",
-    "any"
   };
   
   if (node->tag < GI_TYPE_TAG_ARRAY)
@@ -1437,11 +1448,11 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    type->tag == GI_TYPE_TAG_UTF8 ||
 	    type->tag == GI_TYPE_TAG_FILENAME)
 	  { 
-	    blob->reserved = 0;
-	    blob->reserved2 = 0;
-	    blob->pointer = type->is_pointer;
-	    blob->reserved3 = 0;
-	    blob->tag = type->tag;
+	    blob->flags.reserved = 0;
+	    blob->flags.reserved2 = 0;
+	    blob->flags.pointer = type->is_pointer;
+	    blob->flags.reserved3 = 0;
+	    blob->flags.tag = type->tag;
 	  }
 	else 
 	  {
@@ -1481,11 +1492,11 @@ g_ir_node_build_typelib (GIrNode         *node,
                       array->has_size = type->has_size;
 		      array->reserved2 = 0;
                       if (array->has_length)
-                        array->length = type->length;
+                        array->dimensions.length = type->length;
                       else if (array->has_size)
-                        array->size  = type->size;
+                        array->dimensions.size  = type->size;
                       else
-                        array->length = -1;
+                        array->dimensions.length = -1;
 		      
 		      pos = *offset2 + G_STRUCT_OFFSET (ArrayTypeBlob, type);
 		      *offset2 += sizeof (ArrayTypeBlob);
@@ -1586,8 +1597,6 @@ g_ir_node_build_typelib (GIrNode         *node,
 	FieldBlob *blob;
 
 	blob = (FieldBlob *)&data[*offset];
-	/* We handle the size member specially below, so subtract it */
-	*offset += sizeof (FieldBlob) - sizeof (SimpleTypeBlob);
 
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->readable = field->readable;
@@ -1599,8 +1608,22 @@ g_ir_node_build_typelib (GIrNode         *node,
 	else
 	  blob->struct_offset = 0xFFFF; /* mark as unknown */
 
-        g_ir_node_build_typelib ((GIrNode *)field->type, 
-				 node, build, offset, offset2);
+        if (field->callback)
+          {
+            blob->has_embedded_type = TRUE;
+            blob->type.offset = GI_INFO_TYPE_CALLBACK;
+	    *offset += sizeof (FieldBlob);
+            g_ir_node_build_typelib ((GIrNode *)field->callback,
+			             node, build, offset, offset2);
+          }
+        else
+          {
+            blob->has_embedded_type = FALSE;
+            /* We handle the size member specially below, so subtract it */
+	    *offset += sizeof (FieldBlob) - sizeof (SimpleTypeBlob);
+            g_ir_node_build_typelib ((GIrNode *)field->type,
+				     node, build, offset, offset2);
+          }
       }
       break;
 
