@@ -256,6 +256,17 @@ class GLibTransformer(object):
         except KeyError, e:
             return Unresolved(gtype_name)
 
+    def _resolve_gtypename_chain(self, gtype_names):
+        """Like _resolve_gtypename, but grab the first one that resolves.
+        If none of them do, return an Unresolved for the first."""
+        for gtype_name in gtype_names:
+            try:
+                return self._transformer.gtypename_to_giname(gtype_name,
+                                                             self._names)
+            except KeyError, e:
+                continue
+        return Unresolved(gtype_names[0])
+
     def _execute_binary(self):
         in_path = os.path.join(self._binary.tmpdir, 'types.txt')
         f = open(in_path, 'w')
@@ -397,8 +408,8 @@ class GLibTransformer(object):
             return False
         if func.parameters:
             return False
-        if func.retval.type.name not in ['GLib.Quark',
-                                         'GQuark']:
+        if (func.retval.type.name != 'GLib.Quark' and
+            func.retval.type.ctype != 'GQuark'):
             return False
 
         self._error_quark_functions.append(func)
@@ -547,6 +558,8 @@ class GLibTransformer(object):
         elif record.name in g_internal_names:
             # Avoid duplicates
             return
+        if record.name == 'InitiallyUnownedClass':
+            record.fields = self._names.names['ObjectClass'][1].fields
         node = self._names.names.get(record.name)
         if node is None:
             self._add_attribute(record, replace=True)
@@ -687,8 +700,11 @@ class GLibTransformer(object):
         # to skip it
         if type_name == 'GObject':
             return
-        parent_type_name = xmlnode.attrib['parent']
-        parent_gitype = self._resolve_gtypename(parent_type_name)
+        # Get a list of parents here; some of them may be hidden, and what
+        # we really want to do is use the most-derived one that we know of.
+        #
+        parent_type_names = xmlnode.attrib['parents'].split(',')
+        parent_gitype = self._resolve_gtypename_chain(parent_type_names)
         is_abstract = not not xmlnode.attrib.get('abstract', False)
         node = GLibObject(
             self._transformer.remove_prefix(type_name),
@@ -746,7 +762,7 @@ class GLibTransformer(object):
         for interface in xmlnode.findall('implements'):
             gitype = self._resolve_gtypename(interface.attrib['name'])
             gt_interfaces.append(gitype)
-        node.interfaces = gt_interfaces
+        node.interfaces = sorted(gt_interfaces)
 
     def _introspect_properties(self, node, xmlnode):
         for pspec in xmlnode.findall('property'):
@@ -762,6 +778,7 @@ class GLibTransformer(object):
                 readable, writable, construct, construct_only,
                 ctype,
                 ))
+        node.properties = sorted(node.properties)
 
     def _introspect_signals(self, node, xmlnode):
         for signal_info in xmlnode.findall('signal'):
@@ -781,6 +798,7 @@ class GLibTransformer(object):
                 param.transfer = 'none'
                 signal.parameters.append(param)
             node.signals.append(signal)
+        node.signals = sorted(node.signals)
 
     # Resolver
 
@@ -942,9 +960,9 @@ class GLibTransformer(object):
 
         last_param = func.parameters.pop()
 
-        if (last_param.type.name == 'GLib.Error' or
-            (self._namespace_name == 'GLib' and
-             last_param.type.name == 'Error')):
+        # Checking type.name=='GLib.Error' generates false positives
+        # on methods that take a 'GError *'
+        if last_param.type.ctype == 'GError**':
             func.throws = True
         else:
             func.parameters.append(last_param)
