@@ -2,6 +2,7 @@
 #define __FOO_OBJECT_H__
 
 #include <glib-object.h>
+#include <gio/gio.h> /* GAsyncReadyCallback */
 #include "utility.h"
 
 #define FOO_SUCCESS_INT 0x1138
@@ -87,6 +88,9 @@ struct _FooObjectClass
 
   /* Intended to match GFile */
   void (*read_fn) (FooObject *object, int offset, int length);
+  
+  /* Test reserved stuff */
+  GCallback _reserved[4];
 };
 
 gint                  foo_init                     (void);
@@ -122,6 +126,8 @@ void                  foo_object_read              (FooObject *object, int offse
 
 int                   foo_object_static_meth       (void);
 
+void                  foo_object_skipped_method    (FooObject *object);
+
 struct _FooSubobject
 {
   FooObject parent_instance;
@@ -185,6 +191,13 @@ typedef enum
   FOO_ENUM_FULLNAME_THREE
 } FooEnumFullname;
 
+typedef enum
+{
+  FOO_ADDRESS_INVALID,
+  FOO_ADDRESS_IPV4,
+  FOO_ADDRESS_IPV6
+} FooAddressType;
+
 typedef struct _FooBoxed FooBoxed;
 
 GType                 foo_boxed_get_type       (void) G_GNUC_CONST;
@@ -332,6 +345,14 @@ void foo_test_varargs_callback2 (FooVarargsCallback callback);
 void foo_test_varargs_callback3 (FooVarargsCallback callback,
 				 FooVarargsCallback callback2);
 
+/* Make sure callbacks get the right scope by default */
+void foo_async_ready_callback(GCancellable *cancellable,
+                              GAsyncReadyCallback callback,
+                              gpointer user_data);
+void foo_destroy_notify_callback(FooCallback callback,
+                                 gpointer data,
+                                 GDestroyNotify destroy);
+
 typedef enum {
   FOO_ERROR_GOOD,
   FOO_ERROR_BAD,
@@ -370,4 +391,5 @@ typedef enum {
 } FooSkippable;
 void foo_skip_me (FooSkippable fs);
 
+
 #endif /* __FOO_OBJECT_H__ */
