@@ -2,6 +2,7 @@
 /* GObject introspection: IDL generator
  *
  * Copyright (C) 2005 Matthias Clasen
+ * Copyright (C) 2008,2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,7 @@
 /* FIXME: Avoid global */
 static gchar *output = NULL;
 gchar **includedirs = NULL;
+static gboolean show_all = FALSE;
 
 typedef struct {
   FILE *file;
@@ -316,12 +318,32 @@ write_type_info (const gchar *namespace,
     }
 }
 
+static void
+write_attributes (Xml *file,
+                   GIBaseInfo *info)
+{
+  GIAttributeIter iter = { 0, };
+  char *name, *value;
+
+  while (g_base_info_iterate_attributes (info, &iter, &name, &value))
+    {
+      xml_start_element (file, "attribute");
+      xml_printf (file, " name=\"%s\" value=\"%s\"", name, value);
+      xml_end_element (file, "attribute");
+    }
+}
+
 static void
 write_constant_value (const gchar *namespace, 
 		      GITypeInfo *info,
 		      GArgument *argument,
 		      Xml *file);
 
+static void
+write_callback_info (const gchar    *namespace,
+		     GICallbackInfo *info,
+		     Xml            *file);
+
 static void
 write_field_info (const gchar *namespace,
 		  GIFieldInfo *info,
@@ -333,6 +355,7 @@ write_field_info (const gchar *namespace,
   gint size;
   gint offset;
   GITypeInfo *type;
+  GIBaseInfo *interface;
   GArgument value; 
 
   name = g_base_info_get_name ((GIBaseInfo *)info);
@@ -354,6 +377,8 @@ write_field_info (const gchar *namespace,
   if (size)
     xml_printf (file, " bits=\"%d\"", size);
 
+  write_attributes (file, (GIBaseInfo*) info);
+
   type = g_field_info_get_type (info);
 
   if (branch)
@@ -365,7 +390,21 @@ write_field_info (const gchar *namespace,
       xml_printf (file, "\"");
     }
 
-  write_type_info (namespace, type, file);
+  if (show_all)
+    {
+      if (offset >= 0)
+        xml_printf (file, "offset=\"%d\"", offset);
+    }
+
+  interface = g_type_info_get_interface (type);
+  if (interface && g_base_info_get_type(interface) == GI_INFO_TYPE_CALLBACK)
+    write_callback_info (namespace, (GICallbackInfo *)interface, file);
+  else
+    write_type_info (namespace, type, file);
+
+  if (interface)
+    g_base_info_unref (interface);
+
   g_base_info_unref ((GIBaseInfo *)type);
 
   xml_end_element (file, "field");
@@ -379,6 +418,8 @@ write_callable_info (const gchar    *namespace,
   GITypeInfo *type;
   gint i;
 
+  write_attributes (file, (GIBaseInfo*) info);
+
   type = g_callable_info_get_return_type (info);
 
   xml_start_element (file, "return-value");
@@ -455,6 +496,27 @@ write_callable_info (const gchar    *namespace,
       
       if (g_arg_info_is_optional (arg))
 	xml_printf (file, " optional=\"1\"");
+
+      switch (g_arg_info_get_scope (arg))
+        {
+        case GI_SCOPE_TYPE_INVALID:
+          break;
+        case GI_SCOPE_TYPE_CALL:
+          xml_printf (file, " scope=\"call\"");
+          break;
+        case GI_SCOPE_TYPE_ASYNC:
+          xml_printf (file, " scope=\"async\"");
+          break;
+        case GI_SCOPE_TYPE_NOTIFIED:
+          xml_printf (file, " scope=\"notified\"");
+          break;
+        }
+      
+      if (g_arg_info_get_closure (arg) >= 0)
+        xml_printf (file, " closure=\"%d\"", g_arg_info_get_closure (arg));
+      
+      if (g_arg_info_get_destroy (arg) >= 0)
+        xml_printf (file, " destroy=\"%d\"", g_arg_info_get_destroy (arg));
       
       type = g_arg_info_get_type (arg);
       write_type_info (namespace, type, file);
@@ -542,7 +604,9 @@ write_struct_info (const gchar  *namespace,
   const gchar *type_name;
   const gchar *type_init;
   gboolean deprecated;
+  gboolean is_gtype_struct;
   gint i;
+  gint size;
   int n_elts;
 
   name = g_base_info_get_name ((GIBaseInfo *)info);
@@ -567,7 +631,17 @@ write_struct_info (const gchar  *namespace,
 	  
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
+  
+  is_gtype_struct = g_struct_info_is_gtype_struct (info);
+  if (is_gtype_struct)
+    xml_printf (file, " glib:is-gtype-struct=\"1\"");
+
+  write_attributes (file, (GIBaseInfo*) info);
 	
+  size = g_struct_info_get_size (info);
+  if (show_all && size >= 0)
+    xml_printf (file, " size=\"%d\"", size);
+
   n_elts = g_struct_info_get_n_fields (info) + g_struct_info_get_n_methods (info);
   if (n_elts > 0)
     {
@@ -609,6 +683,8 @@ write_value_info (const gchar *namespace,
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
   
+  write_attributes (file, (GIBaseInfo*) info);
+
   xml_end_element (file, "member");
 }
 
@@ -705,6 +781,8 @@ write_constant_info (const gchar    *namespace,
 
   write_type_info (namespace, type, file);
 
+  write_attributes (file, (GIBaseInfo*) info);
+
   xml_end_element (file, "constant");
   
   g_base_info_unref ((GIBaseInfo *)type);
@@ -739,7 +817,8 @@ write_enum_info (const gchar *namespace,
   
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
-	
+
+  write_attributes (file, (GIBaseInfo*) info);
 
   for (i = 0; i < g_enum_info_get_n_values (info); i++)
     {
@@ -801,6 +880,7 @@ write_vfunc_info (const gchar *namespace,
 {
   GIVFuncInfoFlags flags;
   const gchar *name;
+  GIFunctionInfo *invoker;
   gboolean deprecated;
   gint offset;
 
@@ -808,8 +888,9 @@ write_vfunc_info (const gchar *namespace,
   flags = g_vfunc_info_get_flags (info);
   deprecated = g_base_info_is_deprecated ((GIBaseInfo *)info);
   offset = g_vfunc_info_get_offset (info);
+  invoker = g_vfunc_info_get_invoker (info);
 
-  xml_start_element (file, "vfunc");
+  xml_start_element (file, "virtual-method");
   xml_printf (file, " name=\"%s\"", name);
 
   if (deprecated)
@@ -825,9 +906,12 @@ write_vfunc_info (const gchar *namespace,
     
   xml_printf (file, " offset=\"%d\"", offset);
 
+  if (invoker)
+    xml_printf (file, " invoker=\"%s\"", g_base_info_get_name ((GIBaseInfo*)invoker));
+
   write_callable_info (namespace, (GICallableInfo*)info, file);
 
-  xml_end_element (file, "vfunc");
+  xml_end_element (file, "virtual-method");
 }
 
 static void
@@ -861,7 +945,9 @@ write_property_info (const gchar    *namespace,
 
   if (flags & G_PARAM_CONSTRUCT_ONLY)
     xml_printf (file, " construct-only=\"1\"");
-    
+
+  write_attributes (file, (GIBaseInfo*) info);
+
   type = g_property_info_get_type (info);
 
   write_type_info (namespace, type, file);
@@ -880,6 +966,7 @@ write_object_info (const gchar  *namespace,
   gboolean deprecated;
   gboolean is_abstract;
   GIObjectInfo *pnode;
+  GIStructInfo *class_struct;
   gint i;
 
   name = g_base_info_get_name ((GIBaseInfo *)info);
@@ -897,6 +984,13 @@ write_object_info (const gchar  *namespace,
       write_type_name_attribute (namespace, (GIBaseInfo *)pnode, "parent", file);
       g_base_info_unref ((GIBaseInfo *)pnode);
     }
+  
+  class_struct = g_object_info_get_class_struct (info);
+  if (class_struct)
+    {
+      write_type_name_attribute (namespace, (GIBaseInfo*) class_struct, "glib:type-struct", file);
+      g_base_info_unref ((GIBaseInfo*)class_struct);
+    }
 
   if (is_abstract)
     xml_printf (file, " abstract=\"1\"");
@@ -905,7 +999,8 @@ write_object_info (const gchar  *namespace,
 
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
-	
+
+  write_attributes (file, (GIBaseInfo*) info);
 
   if (g_object_info_get_n_interfaces (info) > 0)
     {
@@ -972,6 +1067,7 @@ write_interface_info (const gchar     *namespace,
   const gchar *name;
   const gchar *type_name;
   const gchar *type_init;
+  GIStructInfo *class_struct;
   gboolean deprecated;
   gint i;
 
@@ -984,9 +1080,18 @@ write_interface_info (const gchar     *namespace,
   xml_printf (file, " name=\"%s\" glib:type-name=\"%s\" glib:get-type=\"%s\"",
 	     name, type_name, type_init);
 
+  class_struct = g_interface_info_get_iface_struct (info);
+  if (class_struct)
+    {
+      write_type_name_attribute (namespace, (GIBaseInfo*) class_struct, "glib:type-struct", file);
+      g_base_info_unref ((GIBaseInfo*)class_struct);
+    }
+
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
 
+  write_attributes (file, (GIBaseInfo*) info);
+
   if (g_interface_info_get_n_prerequisites (info) > 0)
     {
       for (i = 0; i < g_interface_info_get_n_prerequisites (info); i++)
@@ -1068,6 +1173,7 @@ write_union_info (const gchar *namespace,
   const gchar *type_init;
   gboolean deprecated;
   gint i;
+  gint size;
 
   name = g_base_info_get_name ((GIBaseInfo *)info);
   deprecated = g_base_info_is_deprecated ((GIBaseInfo *)info);
@@ -1083,7 +1189,12 @@ write_union_info (const gchar *namespace,
 	  
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
-	
+
+  size = g_union_info_get_size (info);
+  if (show_all && size >= 0)
+    xml_printf (file, " size=\"%d\"", size);
+
+  write_attributes (file, (GIBaseInfo*) info);
 
   if (g_union_info_is_discriminated (info))
     {
@@ -1182,16 +1293,20 @@ write_repository (const char   *namespace,
   if (TRUE)
     {
       const gchar *shared_library;
+      const gchar *c_prefix;
       const char *ns = namespace;
       const char *version;
 
       version = g_irepository_get_version (repository, ns);
 
       shared_library = g_irepository_get_shared_library (repository, ns);
+      c_prefix = g_irepository_get_c_prefix (repository, ns);
       xml_start_element (xml, "namespace");
       xml_printf (xml, " name=\"%s\" version=\"%s\"", ns, version);
       if (shared_library)
         xml_printf (xml, " shared-library=\"%s\"", shared_library);
+      if (c_prefix)
+        xml_printf (xml, " c:prefix=\"%s\"", c_prefix);
       
       for (j = 0; j < g_irepository_get_n_infos (repository, ns); j++)
 	{
@@ -1304,7 +1419,8 @@ main (int argc, char *argv[])
     {
       { "shlib", 0, 0, G_OPTION_ARG_NONE, &shlib, "handle typelib embedded in shlib", NULL },
       { "output", 'o', 0, G_OPTION_ARG_FILENAME, &output, "output file", "FILE" }, 
-      { "includedir", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &includedirs, "include directories in GIR search path", NULL }, 
+      { "includedir", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &includedirs, "include directories in GIR search path", NULL },
+      { "all", 0, 0, G_OPTION_ARG_NONE, &show_all, "show all available information", NULL, },
       { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &input, NULL, NULL },
       { NULL, }
     };
